
from .listener import Listener
from .sender import StartConversationMessage, SendTextMessage


class Client:

    def __init__(self, bot_api_url, listener_host: str = "127.0.0.1", listener_port: int = 5789):

        # Save the urls
        self.bot_api_url = bot_api_url
        self.listener_host = listener_host
        self.listener_port = listener_port

        # Launch the listener server
        self.listener = Listener(listener_host, listener_port)
        self.listener.start()

    async def start_conversation(self, user_id: int) -> str:

        return await StartConversationMessage(
            self.bot_api_url, self.listener_host, self.listener_port, {
                'user_id': user_id
            }).send()

    async def send_message(self, user_id: int, text: str) -> str:

        return await SendTextMessage(
            self.bot_api_url, self.listener_host, self.listener_port, {
                'user_id': user_id,
                'text': text
            }).send()



# Always prefer setuptools over distutils
from setuptools import setup, find_packages

# To use a consistent encoding
from codecs import open
from os import path

# The directory containing this file
HERE = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(HERE, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

# This call to setup() does all the work.
setup(
    name="botbuilder-discord",
    version="0.1.0",
    description="Unofficial Discord.py connector for Microsoft's Bot Framework",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://botbuilder-discord.readthedocs.io/",
    author="Joffrey Bienvenu",
    author_email="joffreybvn@gmail.com",
    license="MIT",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Operating System :: OS Independent",
        "Topic :: Utilities"
    ],
    packages=find_packages(include=["botbuilder_discord", "botbuilder_discord.*"]),
    include_package_data=True,
    install_requires=["aiohttp", "Flask"]
)




from typing import List, Callable


class ResponseQueue:
    """
    Global object to store the responses given by the listener, and to send
    them to the Client and Message objects.
    """

    def __init__(self):
        self._observers: List[Callable] = []
        self._entries = {}

    def append(self, key: str, content: dict):
        """
        Called by the listener. Add a response to the _entries
        dictionary. Trigger the observers.

        :param key: The key of the new entry in the dictionary. This key is
            an unique message id.
        :param content: The content stored with this new key, in the
            dictionary. This content is another dict containing the response
            message.
        """

        # Append to the dictionary
        self._entries[key] = content

        # Trigger all the observers' functions.
        for callback in self._observers:
            callback(self._entries)

    def bind_to(self, callback: Callable):
        """
        Add a callback function to the observer list.

        :param callback: A function triggered when a new message is appended
            to the _entries dictionary.
        """
        self._observers.append(callback)

    def unbind(self, callback: Callable):
        """
        Unbind (remove) a given callback function from the observers list.
        """
        self._observers.remove(callback)


# Instantiate a global response queue
response_queue = ResponseQueue()
